// Reprsentiert ein Chromosom 
// Chromosomseintrgen (Gene) sind hier Integerwerte

package ga;

import java.util.BitSet;


/**
 ** Diese Klasse reprsentiert ein einzelnens Chromosom, welches durch
 ** einen BitString dargestellt wird
 **
 ** 
 */
public class Chromosom_bit
{
   BitSet chrom;                 // Genotype - bit string
   int laenge;
      
  
  /**
   ** Dieser Konstruktor erzeugt ein neues Chromosom.
   **
   */
  
   public Chromosom_bit(BitSet chrom, int laenge)
   {
      // erzeugen eines neuen Chromosoms durch kopieren
      this.chrom = (BitSet) chrom.clone(); 
      this.laenge = laenge;
   
   }

  
  /**
   ** Dieser Konstruktor erzeugt ein neues Chromosom.
   **
   */
  
   public Chromosom_bit(Chromosom_bit chrom)
   {
      // erzeugen eines neuen Chromosoms durch kopieren
      this.chrom = (BitSet) chrom.chrom().clone(); 
	  this.laenge = chrom.laenge();
   }


  /**
   ** Dieser Konstruktor eines Chromosoms erzeugt unter Verwendung
   ** von 2 anderen Chromosomen ein neues Chromosom.
   ** Das neue Chromosom (Kind) wird durch Crossover und Mutation
   ** der beiden Eltern erzeugt. 
   ** Die Reihenfolge, in der Vater und Mutter angegeben werden,
   ** entscheidet folglich wie das Kind aussieht.
   */

   public Chromosom_bit(Chromosom_bit parent1, Chromosom_bit parent2)
   {      
      // Ereugt unter Verwendung der Beiden Eltern ein neuens Chromosom
      chrom = new BitSet(parent1.laenge());
      laenge = parent1.laenge();
      crossover(parent1, parent2);
      //mutiere();
   }



   //_____________________________________________________________________
   // gibt die Laenge des Chromosoms zurck	
   public int laenge ()
   {      
      return laenge;
   }
 

  

  /**
   ** This method performs the GA crossover operation.  With pcross
   ** probability it selects a crossover site at random and takes the
   ** bits up to that site from parent 1 and from that site to the end
   ** of the chromosome of parent 2. This method implements simple single
   ** point crossover.  If no crossover is done the resulting individual
   ** will have the same chromosome as parent 1.
   **
   ** @param aParent1 The parent from which we will take the left part up
   **        to the cross site of the chromosome if crossover is performed.
   ** @param aParent2 The parent from which we will take the right part from
   **        the cross site of the chromosome if crossover is performed.
   ** @return void
   ** @exception
   */
   void crossover(Chromosom_bit parent1, Chromosom_bit parent2)
   {
      int i;
      int c_punkt;

      // Crossoverstelle
      c_punkt = GA.c_punkt(laenge);
      
      System.out.println("Crossover bei C1 oder 3: Crossoverstelle: " + c_punkt);
      
      // Teil von parent1
      for (i=0; i<c_punkt; i++)
      {
         if (parent1.chrom.get(i))
            chrom.set(i);
         else
            chrom.clear(i);
      }

      // Teil von parent2
      for (i=c_punkt; i<laenge; i++)
      {
         if (parent2.chrom.get(i))
            chrom.set(i);
         else
            chrom.clear(i);
      }
   }


  /**
   ** This method performs the GA mutation operation.
   **
   ** @param 
   ** @return void
   ** @exception
   */
   void mutiere()
   {
      int i;

      for (i=0; i<laenge; i++)
      {
         if (GA.mutiere_C13(laenge) )
         {
         	System.out.println("   Mutiere c1 oder c3");
            if (chrom.get(i))
               chrom.clear(i);
            else
               chrom.set(i);
         }
      }
   }


 
  /**
   ** This accessor method returns the chromosome of this Individual
   **
   ** @param 
   ** @return A BitSet which is this Individuals chromosome
   ** @exception
   */
   public BitSet chrom()
   {	
      return chrom;
   }


  
/**
   ** Clean up after this Individual to help in garbage collection.
   **
   ** @param
   ** @return void
   ** @exception
   */
   protected void finalize()
   {
      chrom = null;
   }


  

}
